@@ -16,7 +16,7 @@ class NeuralNetwork():
         self.activation_fucntions_deriv = []
         self.hidden_layers = 0
         self.learning_rate = 0.05
-    def add(self, units=1, dropout=0, activation="sigmoid", activation_deriv="none"):
+    def add(self, units=1, dropout=0.0, activation="sigmoid", activation_deriv="none"):
         self.each_layer_list.append([units,dropout,activation, activation_deriv])
     def set_learning_rate(self, lr):
         self.learning_rate = lr
@@ -90,6 +90,28 @@ class NeuralNetwork():
         data = self.forward_method(x)
         error = y - data
         return np.mean(np.abs(error)).sum()
+    def dropout(self, dropout_size, weights, nr):
+        all_weights = []
+        counter = weights.shape[1] * dropout_size
+        while counter > 0:
+            for x in range(weights.shape[1]):
+                if self.ai_system_list[nr].dropout_list[x] == 1:
+                    for xs in range(weights.shape[0]):
+                        weights[xs][x] = 0
+                        counter -= 1
+        all_weights.append(weights)
+        all_weights = np.array(weights)
+        return all_weights
+    def select_droupout_neurons(self):
+        for i in range(self.hidden_layers):
+            if self.ai_system_list[i].dropout > 0:
+                self.ai_system_list[i].dropout_list = np.zeros(len(self.ai_system_list[i].dropout_list))
+                counter = self.ai_system_list[i].weights.shape[1] * self.ai_system_list[i].dropout
+                while counter > 0:
+                    random_neuron = np.random.randint(0, self.ai_system_list[i].weights.shape[1])
+                    if self.ai_system_list[i].dropout_list[random_neuron] == 0:
+                        self.ai_system_list[i].dropout_list[random_neuron] = 1
+                        counter -= 1
     def train(self, x=None, y=None, x_test=None, y_test=None, epochs=1000, info=1, shake_data=False, batch_size=10):
         self.prepare_network()
         if len(self.each_layer_list) <= 1:
@@ -106,12 +128,16 @@ class NeuralNetwork():
         if self.ai_system_list[-1].weights.shape[1] != y[0].shape[1]:
             print(f"TechAI.Error: Outputs {self.ai_system_list[-1].weights.shape[1]} - OutputData {y[0].shape[1]}")
         for train in range(epochs):
+            self.select_droupout_neurons()
             total_error = 0
             total_error_test = 0
             for nr in range(len(x)):
                 forward = x[nr]
                 for i in range(self.hidden_layers):
                     forward = self.activation_fucntions[i + 1](np.dot(forward,self.ai_system_list[i].weights))
+                    #self.ai_system_list[i].forward = forward
+                    if self.ai_system_list[i].dropout > 0:
+                        forward = self.dropout(self.ai_system_list[i].dropout, forward, i)
                     self.ai_system_list[i].forward = forward
 
                 self.ai_system_list[-1].error = y[nr] - self.ai_system_list[-1].forward
@@ -121,6 +147,8 @@ class NeuralNetwork():
                     ii = i + 1
                     self.ai_system_list[-ii].error = np.dot(self.ai_system_list[-i].delta, self.ai_system_list[-i].weights.T)
                     self.ai_system_list[-ii].delta = self.ai_system_list[-ii].error * self.activation_fucntions_deriv[-ii](self.ai_system_list[-ii].forward)
+                    if self.ai_system_list[-ii].dropout > 0:
+                        self.ai_system_list[-ii].delta = self.dropout(self.ai_system_list[-ii].dropout, self.ai_system_list[-ii].delta, -ii)
                 total_error += np.mean(np.abs(self.ai_system_list[-1].error)).sum() / len(x)
                 self.ai_system_list[0].weights += self.learning_rate * np.dot(x[nr].T, self.ai_system_list[0].delta)
                 for i in range(1, self.hidden_layers):
@@ -140,4 +168,5 @@ class AI_SYSTEM():
         self.input = n_out
         self.dropout = dropout
         self.activation = activation
-        self.activation_deriv = activation_deriv
\ No newline at end of file
+        self.activation_deriv = activation_deriv
+        self.dropout_list = np.zeros(n_out)
\ No newline at end of file
